.g-thumbnail-size-input{border-radius:3px;padding:5px 10px;font-size:12px;border:1px solid #ccc;width:90px;vertical-align:middle}.g-thumbnail-size-between{margin:0 9px;vertical-align:middle;font-size:14px}.g-thumbnail-crop-container{margin-left:20px}.g-thumbnail-size-form-container{margin-bottom:15px}.g-thumbnail-custom-target-container{margin-bottom:8px}.g-thumbnail-custom-target-container .g-target-result{padding:5px 10px;background-color:#eee;border-radius:3px;margin-bottom:6px}.g-thumbnail-container{display:inline-block;margin:0 10px 10px 0;vertical-align:top;position:relative}.g-thumbnail-container .g-thumbnail-actions-container{border-radius:3px;background-color:#fffc;padding:2px 4px;position:absolute;top:4px;right:4px;font-size:12px;display:inline-block;opacity:0;transition:opacity .4s}.g-thumbnail-container .g-thumbnail-actions-container.g-show{opacity:1}.g-thumbnail-container .g-thumbnail-actions-container a{color:#333}.g-thumbnails-header-container{margin-bottom:8px}
